@@ -823,9 +823,6 @@ def _extract_prop_type(props, schema, propname, subschema, is_pattern):
     if propname.startswith('$'):
         return
 
-    if subschema.keys() & {'properties', 'patternProperties', 'additionalProperties'}:
-        _extract_subschema_types(props, schema, subschema)
-
     # We only support local refs
     if '$ref' in subschema and subschema['$ref'].startswith('#/'):
         sch_path = subschema['$ref'].split('/')[1:]
@@ -886,35 +883,40 @@ def _extract_prop_type(props, schema, propname, subschema, is_pattern):
     else:
         dim = ((0, 0), (0, 0))
 
-    if propname in props:
-        dup_prop = None
-        for p in props[propname]:
-            if p['type'] is None:
-                dup_prop = p
-                break
-            if dim != ((0, 0), (0, 0)) and (p['type'] == 'phandle-array' or p['type'].endswith('-matrix')):
-                if not 'dim' in p:
-                    p['dim'] = dim
-                elif p['dim'] != dim:
-                    # Conflicting dimensions
-                    p['dim'] = _merge_dim(p['dim'], dim)
-                return
-            if p['type'].startswith(prop_type):
-                # Already have the same or looser type
-                if schema['$id'] not in p['$id']:
-                    p['$id'] += [schema['$id']]
-                return
-            elif p['type'] in prop_type:
-                # Replace scalar type with array type
-                new_prop['$id'] += p['$id']
-                dup_prop = p
-                break
+    dup_prop = None
+    for p in props[propname]:
+        if p['type'] is None:
+            dup_prop = p
+            break
+        if dim != ((0, 0), (0, 0)) and (p['type'] == 'phandle-array' or p['type'].endswith('-matrix')):
+            if not 'dim' in p:
+                p['dim'] = dim
+            elif p['dim'] != dim:
+                # Conflicting dimensions
+                p['dim'] = _merge_dim(p['dim'], dim)
+            return
+        if p['type'].startswith(prop_type):
+            # Already have the same or looser type
+            if schema['$id'] not in p['$id']:
+                p['$id'] += [schema['$id']]
+                # Descend into child schemas if we haven't seen this node already
+                if prop_type == 'node':
+                    break
+            return
+        elif p['type'] in prop_type:
+            # Replace scalar type with array type
+            new_prop['$id'] += p['$id']
+            dup_prop = p
+            break
 
-        if dup_prop:
-            props[propname].remove(dup_prop)
+    if dup_prop:
+        props[propname].remove(dup_prop)
 
     props[propname] += [new_prop]
 
+    if subschema.keys() & {'properties', 'patternProperties', 'additionalProperties'}:
+        _extract_subschema_types(props, schema, subschema)
+
 
 def _extract_subschema_types(props, schema, subschema):
     if not isinstance(subschema, dict):
